/**
 * Created on 2007-1-31
 * 
 * Copyright (c) Sybase, Inc. 2004-2007. All rights reserved.
 */
package org.eclipse.datatools.enablement.sybase;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.Iterator;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.swt.graphics.Image;

/**
 * @author renj
 * 
 */
public class SybaseImages
{
    private static final String         NAME_PREFIX          = "org.eclipse.datatools.enablement.sybase"; //$NON-NLS-1$
    private static final int            NAME_PREFIX_LENGTH   = NAME_PREFIX.length();
    private static URL                  baseURL              = null;

    // The plugin registry
    private static ImageRegistry        imageRegistry        = null;
    private static HashMap              avoidSWTErrorMap     = new HashMap();

    static
    {
        String pathSuffix = "icons/"; //$NON-NLS-1$
        baseURL = Activator.getDefault().getBundle().getEntry(pathSuffix);
    }

    public static final String          IMG_SYSTEMPROCEDURE  = NAME_PREFIX + "system_procedure.gif"; //$NON-NLS-1$
    public static final ImageDescriptor DESC_SYSTEMPROCEDURE = createManaged(null, IMG_SYSTEMPROCEDURE);
    public static final String          IMG_SYSTEMTABLE      = NAME_PREFIX + "system_table.gif"; //$NON-NLS-1$
    public static final ImageDescriptor DESC_SYSTEMTABLE     = createManaged(null, IMG_SYSTEMTABLE);
    public static final String          IMG_SYSTEMVIEW       = NAME_PREFIX + "system_view.gif"; //$NON-NLS-1$
    public static final ImageDescriptor DESC_SYSTEMVIEW      = createManaged(null, IMG_SYSTEMVIEW);
    
    // Folder
    public static final String          IMG_FOLDER           = NAME_PREFIX + "folder.gif"; //$NON-NLS-1$
    public static final ImageDescriptor DESC_FOLDER          = createManaged(null, IMG_FOLDER);

    // Primary key
    public static final String          IMG_PK               = NAME_PREFIX + "primaryKey.gif"; //$NON-NLS-1$
    public static final ImageDescriptor DESC_PK              = createManaged(null, IMG_PK);

    // Foreign key
    public static final String          IMG_FK               = NAME_PREFIX + "foreignKey.gif"; //$NON-NLS-1$
    public static final ImageDescriptor DESC_FK              = createManaged(null, IMG_FK);

    // Unqiue constraint
    public static final String          IMG_UNIQUE           = NAME_PREFIX + "unique_constraint.gif"; //$NON-NLS-1$
    public static final ImageDescriptor DESC_UNIQUE          = createManaged(null, IMG_UNIQUE);

    // Check constraint
    public static final String          IMG_CK               = NAME_PREFIX + "check_constraint.gif"; //$NON-NLS-1$
    public static final ImageDescriptor DESC_CK              = createManaged(null, IMG_CK);

    // User
    public static final String          IMG_USER             = NAME_PREFIX + "user.gif"; //$NON-NLS-1$
    public static final ImageDescriptor DESC_USER            = createManaged(null, IMG_USER);

    // Group
    public static final String          IMG_GROUP            = NAME_PREFIX + "dir.gif"; //$NON-NLS-1$
    public static final ImageDescriptor DESC_GROUP           = createManaged(null, IMG_GROUP);
    
    // Role
    public static final String          IMG_ROLE             = NAME_PREFIX + "role.gif"; //$NON-NLS-1$
    public static final ImageDescriptor DESC_ROLE            = createManaged(null, IMG_GROUP);
    
    /**
     * Returns the image managed under the given key in this registry.
     * 
     * @param key the image's key
     * @return the image managed under the given key
     */
    public static Image get(String key)
    {
        return getImageRegistry().get(key);
    }

    /*
     * Helper method to access the image registry from the JDIDebugUIPlugin class.
     */
    static ImageRegistry getImageRegistry()
    {
        if (imageRegistry == null)
        {
            imageRegistry = new ImageRegistry();
            for (Iterator iter = avoidSWTErrorMap.keySet().iterator(); iter.hasNext();)
            {
                String key = (String) iter.next();
                imageRegistry.put(key, (ImageDescriptor) avoidSWTErrorMap.get(key));
            }
            avoidSWTErrorMap = null;
        }
        return imageRegistry;
    }

    private static ImageDescriptor createManaged(String prefix, String name)
    {
        try
        {
            ImageDescriptor result = ImageDescriptor.createFromURL(makeIconFileURL(prefix, name
                    .substring(NAME_PREFIX_LENGTH)));

            avoidSWTErrorMap.put(name, result);
            if (imageRegistry != null)
            {
                // _logger.error("RepServerImages.registryError"); //$NON-NLS-1$
            }
            return result;
        }
        catch (MalformedURLException e)
        {
            // _logger.error("RepServerImages.malformedURLException", name, e);
            return ImageDescriptor.getMissingImageDescriptor();
        }

    }

    private static URL makeIconFileURL(String prefix, String name) throws MalformedURLException
    {
        if (baseURL == null)
        {
            throw new MalformedURLException();
        }
        StringBuffer buffer = new StringBuffer();
        if (prefix != null)
        {
            buffer = new StringBuffer(prefix);
            buffer.append('/');
        }
        buffer.append(name);
        return new URL(baseURL, buffer.toString());
    }
}
