package org.eclipse.datatools.enablement.sybase.models.sybasesqlmodel.util;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.datatools.connectivity.sqm.core.containment.ContainmentServiceImpl;
import org.eclipse.datatools.modelbase.sql.accesscontrol.AuthorizationIdentifier;
import org.eclipse.datatools.modelbase.sql.accesscontrol.Privilege;
import org.eclipse.datatools.modelbase.sql.schema.Database;
import org.eclipse.datatools.modelbase.sql.schema.SQLObject;

/**
 * Privilege related utilities
 * 
 * @author Dafan Yang
 */
public class PrivilegesUtil
{
    /**
     * Returns all issued privileges of the given SQL object
     * 
     * @param object the SQL object
     * @return all issued privileges
     */
    public static List getSQLObjectPrivileges(SQLObject object)
    {
        List privileges = new ArrayList();
        Database database = (Database) ContainmentServiceImpl.INSTANCE.getRootElement(object);
        Iterator iter = database.getAuthorizationIds().iterator();
        while (iter.hasNext())
        {
            AuthorizationIdentifier authid = (AuthorizationIdentifier) iter.next();
            List receivedPrivileges = getReceivedSQLObjectPrivileges(authid, object);
            privileges.addAll(receivedPrivileges);
        }
        return privileges;
    }

    /**
     * Returns all privileges of the given SQL object owned by the given authid
     * 
     * @param authid
     * @param obj
     * @return
     */
    public static List getReceivedSQLObjectPrivileges(AuthorizationIdentifier authid, SQLObject obj)
    {
        List matchedPrivileges = new ArrayList();
        Iterator iter = authid.getReceivedPrivilege().iterator();
        while (iter.hasNext())
        {
            Privilege p = (Privilege) iter.next();
            if (p.getObject() == obj)
            {
                matchedPrivileges.add(p);
            }
        }
        return matchedPrivileges;
    }

    /**
     * Returns the privilege of the given SQL object owned by the given authid with action specified
     * 
     * @param authid
     * @param obj
     * @param action
     * @return
     */
    public static Privilege getPrivilege(AuthorizationIdentifier authid, SQLObject obj, String action)
    {
        List matchedPrivileges = getReceivedSQLObjectPrivileges(authid, obj);
        Privilege matchedP = null;
        Iterator iter = matchedPrivileges.iterator();
        while (iter.hasNext())
        {
            Privilege p = (Privilege) iter.next();
            if (p.getAction().equals(action))
            {
                matchedP = p;
                break;
            }
        }
        return matchedP;
    }
}
